property pSprite, pStart, pActive, pCompleteCycles, pFadeDiff, pFaded, pAuto, pFadeMax, pFadeMin, pCycles, pPeriodBase, pPeriod

on getBehaviorDescription me
  return "FADE IN/OUT" & RETURN & RETURN & "Gives the appearance of a sprite fading in or out. " & "Choose whether the sprite should first appear at maximum (faded in) or minimum (faded out) values, when the fading should start, the minimum and maximum fade values, the number of times it should fade, and how fast it should fade. " & "The fade can be activated automatically in the first frame, by a click on the sprite, or by sending the sprite an mFadeActivate message." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "animated GIF, bitmap, Flash, text, vector shape" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Fade in or out?" & RETURN & "* Maximum fade value" & RETURN & "* Minimum fade value" & RETURN & "* Fade activation (automatic, click, or message)" & RETURN & "* Number of fade cycles" & RETURN & "* Time period for fade in seconds" & RETURN & RETURN & "Set the number of fade cycles to -1 if you want an endless loop, or to 0 if the fade should happen only once."
end

on getBehaviorTooltip me
  return "Fades a sprite between two fade values once, multiple times, or indefinitely. " & "The fade can be initiated automatically, by mouse click, or via a message to the sprite."
end

on beginSprite me
  pFaded = resolve(pFaded)
  pAuto = resolve(pAuto)
  mInitialize(me)
end

on resolve prop
  case prop of
    pFaded:
      choiceslist = ["In", "Out"]
      lookup = [#in, #out]
    pAuto:
      choiceslist = ["Automatic", "Click", "Message"]
      lookup = [#automatic, #click, #message]
  end case
  return lookup[findPos(choiceslist, prop)]
end

on prepareFrame me
  mUpdate(me)
end

on mouseUp me
  if pAuto = #click then
    mActivate(me)
  end if
end

on mInitialize me
  pSprite = sprite(me.spriteNum)
  pActive = #off
  pCompleteCycles = 0.5
  pPeriod = pPeriodBase * 1000
  if pFadeMax < pFadeMin then
    vMax = pFadeMax
    pFadeMax = pFadeMin
    pFadeMin = vMax
  end if
  pFadeDiff = pFadeMax - pFadeMin
  if pFaded = #in then
    pSprite.blend = pFadeMin
  else
    pSprite.blend = pFadeMax
  end if
  if pAuto = #automatic then
    mActivate(me)
  end if
end

on mUpdate me
  if pActive <> #off then
    vMillis = the milliSeconds
    vElapsed = vMillis - pStart
    if vElapsed >= pPeriod then
      case pActive of
        #in:
          pSprite.blend = pFadeMax
        #out:
          pSprite.blend = pFadeMin
      end case
      pActive = #off
      mCheckCycle(me)
    else
      vFadeDiff = integer(pFadeDiff * vElapsed / pPeriod)
      case pActive of
        #in:
          pSprite.blend = pFadeMin + vFadeDiff
        #out:
          pSprite.blend = pFadeMax - vFadeDiff
      end case
    end if
  end if
end

on mActivate me
  if pFadeDiff then
    case pFaded of
      #in:
        mFadeIn(me)
      #out:
        mFadeOut(me)
    end case
  end if
end

on mCheckCycle me
  vContinue = 0
  case pCycles of
    (-1):
      vContinue = 1
    0:
      vContinue = 0
    otherwise:
      pCompleteCycles = pCompleteCycles + 0.5
      if integer(pCompleteCycles) <= pCycles then
        vContinue = 1
      end if
  end case
  if vContinue then
    if pSprite.blend = pFadeMax then
      mFadeOut(me)
    else
      mFadeIn(me)
    end if
  end if
end

on mFadeIn me
  mFade(me, #in)
end

on mFadeOut me, vTarget
  mFade(me, #out)
end

on mFade me, vInOut
  pActive = vInOut
  pStart = the milliSeconds
end

on mFadeActivate me
  if pAuto = #message then
    mActivate(me)
  end if
end

on isOKToAttach me, aSpriteType, aSpriteNum
  case aSpriteType of
    #graphic:
      return getPos([#animGif, #bitmap, #flash, #text, #vectorShape], sprite(aSpriteNum).member.type) <> 0
    #script:
      return 0
  end case
end

on getPropertyDescriptionList me
  vPDList = [:]
  setaProp(vPDList, #pFaded, [#comment: "Fade in or out?", #format: #string, #default: "In", #range: ["In", "Out"]])
  setaProp(vPDList, #pFadeMax, [#comment: "Maximum Fade Value", #format: #integer, #default: 100, #range: [#min: 0, #max: 100]])
  setaProp(vPDList, #pFadeMin, [#comment: "Minimum Fade Value", #format: #integer, #default: 0, #range: [#min: 0, #max: 100]])
  setaProp(vPDList, #pAuto, [#comment: "Start automatically, when clicked, or by message?", #format: #string, #default: "Automatic", #range: ["Automatic", "Click", "Message"]])
  setaProp(vPDList, #pCycles, [#comment: "Fade cycles (0 = one fade only, -1 = repeat forever)", #format: #integer, #default: 0, #range: [#min: -1, #max: 10]])
  setaProp(vPDList, #pPeriodBase, [#comment: "Time period for fade (seconds)", #format: #float, #default: 2.0, #range: [#min: 0.25, #max: 15.0]])
  return vPDList
end
